#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
from pyams_content.interfaces import MANAGE_CONTENT_PERMISSION
from pyams_utils.interfaces import VIEW_SYSTEM_PERMISSION

__docformat__ = 'restructuredtext'


# import standard library

# import interfaces
from pyams_content.component.extfile.interfaces import IExtFileContainer, IExtFileContainerTarget, IBaseExtFile, IExtFile, \
    IExtImage
from pyams_file.interfaces import IFileInfo
from pyams_i18n.interfaces import INegotiator, II18n
from pyams_skin.interfaces.viewlet import IWidgetTitleViewletManager
from pyams_skin.layer import IPyAMSLayer
from z3c.form.interfaces import NOT_CHANGED

# import packages
from pyams_content.component.extfile import EXTERNAL_FILES_FACTORIES
from pyams_content.component.extfile.zmi.container import ExtFileContainerView
from pyams_form.form import AJAXAddForm, AJAXEditForm
from pyams_form.security import ProtectedFormObjectMixin
from pyams_pagelet.pagelet import pagelet_config
from pyams_skin.viewlet.toolbar import ToolbarAction
from pyams_utils.registry import query_utility
from pyams_utils.traversing import get_parent
from pyams_viewlet.viewlet import viewlet_config
from pyams_zmi.form import AdminDialogAddForm, AdminDialogEditForm
from pyramid.view import view_config
from z3c.form import field
from zope.interface import Interface
from zope.schema import Choice

from pyams_content import _


#
# External file view
#

class IExtFileFactoryChooser(Interface):
    """External file factory chooser interface"""

    factory = Choice(title=_("External file type"),
                     vocabulary='PyAMS files factories',
                     default='file',
                     required=True)


@viewlet_config(name='add-extfile.menu', context=IExtFileContainerTarget, view=ExtFileContainerView,
                layer=IPyAMSLayer, manager=IWidgetTitleViewletManager, weight=50)
class ExtFileAddMenu(ProtectedFormObjectMixin, ToolbarAction):
    """External file add menu"""

    label = _("Add external file")

    url = 'add-extfile.html'
    modal_target = True


@pagelet_config(name='add-extfile.html', context=IExtFileContainerTarget, layer=IPyAMSLayer,
                permission=MANAGE_CONTENT_PERMISSION)
class ExtFileAddForm(AdminDialogAddForm):
    """External file add form"""

    legend = _("Add new external file")
    icon_css_class = 'fa fa-fw fa-file-text-o'

    fields = field.Fields(IExtFileFactoryChooser) + \
             field.Fields(IExtFile).omit('__parent__', '__name__')

    @property
    def ajax_handler(self):
        origin = self.request.params.get('origin')
        if origin == 'link':
            return 'add-extfile-link.json'
        else:
            return 'add-extfile.json'

    edit_permission = MANAGE_CONTENT_PERMISSION

    def updateWidgets(self, prefix=None):
        super(ExtFileAddForm, self).updateWidgets(prefix)
        self.widgets['description'].label_css_class = 'textarea'

    def create(self, data):
        factory = EXTERNAL_FILES_FACTORIES.get(data.get('factory'))
        if factory is not None:
            return factory[0]()

    def update_content(self, content, data):
        data['factory'] = NOT_CHANGED
        return super(ExtFileAddForm, self).update_content(content, data)

    def add(self, object):
        IExtFileContainer(self.context)['none'] = object
        i18n = query_utility(INegotiator)
        if i18n is not None:
            lang = i18n.server_language
            data = II18n(object).get_attribute('data', lang, self.request)
            if data:
                info = IFileInfo(data)
                info.title = II18n(object).get_attribute('title', lang, self.request)
                info.description = II18n(object).get_attribute('description', lang, self.request)
            for lang, data in object.data.items():
                if data is not None:
                    IFileInfo(data).language = lang


@view_config(name='add-extfile.json', context=IExtFileContainerTarget, request_type=IPyAMSLayer,
             permission=MANAGE_CONTENT_PERMISSION, renderer='json', xhr=True)
class ExtFileAJAXAddForm(AJAXAddForm, ExtFileAddForm):
    """External file add form, JSON renderer"""

    def get_ajax_output(self, changes):
        return {'status': 'reload',
                'location': '#external-files.html'}


@view_config(name='add-extfile-link.json', context=IExtFileContainerTarget, request_type=IPyAMSLayer,
             permission=MANAGE_CONTENT_PERMISSION, renderer='json', xhr=True)
class ExtFileLinkAJAXAddForm(AJAXAddForm, ExtFileAddForm):
    """External file link add form, JSON renderer"""

    def get_ajax_output(self, changes):
        target = get_parent(self.context, IExtFileContainerTarget)
        container = IExtFileContainer(target)
        files = [{'id': file.__name__,
                  'text': II18n(file).query_attribute('title', request=self.request)}
                 for file in container.values()]
        return {'status': 'callback',
                'callback': 'PyAMS_content.extfiles.refresh',
                'options': {'files': files,
                            'new_file': {'id': changes.__name__,
                                         'text': II18n(changes).query_attribute('title', request=self.request)}}}


@pagelet_config(name='properties.html', context=IExtFile, layer=IPyAMSLayer, permission=VIEW_SYSTEM_PERMISSION)
class ExtFilePropertiesEditForm(AdminDialogEditForm):
    """External file properties edit form"""

    legend = _("Update file properties")
    icon_css_class = 'fa fa-fw fa-file-text-o'
    dialog_class = 'modal-large'

    fields = field.Fields(IExtFile).omit('__parent__', '__file__')
    ajax_handler = 'properties.json'
    edit_permission = MANAGE_CONTENT_PERMISSION

    def updateWidgets(self, prefix=None):
        super(ExtFilePropertiesEditForm, self).updateWidgets(prefix)
        self.widgets['description'].label_css_class = 'textarea'

    def update_content(self, content, data):
        changes = super(ExtFilePropertiesEditForm, self).update_content(content, data)
        if changes:
            i18n = query_utility(INegotiator)
            if i18n is not None:
                lang = i18n.server_language
                data = II18n(content).get_attribute('data', lang, self.request)
                if data:
                    info = IFileInfo(data)
                    info.title = II18n(content).get_attribute('title', lang, self.request)
                    info.description = II18n(content).get_attribute('description', lang, self.request)
            for lang, data in content.data.items():
                if data and not IFileInfo(data).language:
                    IFileInfo(data).language = lang
        return changes


@pagelet_config(name='properties.html', context=IExtImage, layer=IPyAMSLayer, permission=VIEW_SYSTEM_PERMISSION)
class ExtImagePropertiesEditForm(ExtFilePropertiesEditForm):
    """External image properties edit form"""

    legend = _("Update image properties")
    icon_css_class = 'fa fa-fw fa-picture-o'

    fields = field.Fields(IExtImage).omit('__parent__', '__name__')


@view_config(name='properties.json', context=IExtFile, request_type=IPyAMSLayer,
             permission=MANAGE_CONTENT_PERMISSION, renderer='json', xhr=True)
class ExtFilePropertiesAJAXEditForm(AJAXEditForm, ExtFilePropertiesEditForm):
    """External file properties edit form, JSON renderer"""

    def get_ajax_output(self, changes):
        if ('title' in changes.get(IBaseExtFile, ())) or \
           ('data' in changes.get(IExtFile, ())):
            return {'status': 'reload',
                    'location': '#external-files.html'}
        else:
            return super(ExtFilePropertiesAJAXEditForm, self).get_ajax_output(changes)
