#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'


# import standard library
import json

# import interfaces
from pyams_content.interfaces import MANAGE_CONTENT_PERMISSION
from pyams_content.component.extfile.interfaces import IExtFileLinksContainerTarget
from pyams_content.component.gallery.interfaces import IGalleryLinksContainerTarget
from pyams_content.component.links.interfaces import ILinkLinksContainerTarget
from pyams_content.component.paragraph.interfaces import IParagraphContainerTarget, IParagraphContainer
from pyams_i18n.interfaces import II18n
from pyams_skin.interfaces import IInnerPage, IPageHeader
from pyams_skin.layer import IPyAMSLayer
from pyams_utils.interfaces import VIEW_SYSTEM_PERMISSION
from pyams_zmi.interfaces.menu import IPropertiesMenu
from pyams_zmi.layer import IAdminLayer
from z3c.table.interfaces import IColumn, IValues

# import packages
from pyams_content.shared.common.zmi import WfModifiedContentColumnMixin
from pyams_form.security import ProtectedFormObjectMixin
from pyams_pagelet.pagelet import pagelet_config
from pyams_skin.page import DefaultPageHeaderAdapter
from pyams_skin.table import BaseTable, I18nColumn, TrashColumn, ActionColumn
from pyams_skin.viewlet.menu import MenuItem
from pyams_template.template import template_config
from pyams_utils.adapter import adapter_config, ContextRequestViewAdapter
from pyams_utils.url import absolute_url
from pyramid.view import view_config
from pyams_viewlet.viewlet import viewlet_config
from pyams_zmi.view import AdminView
from pyramid.decorator import reify
from z3c.table.column import GetAttrColumn
from zope.interface import implementer

from pyams_content import _


@viewlet_config(name='paragraphs.menu', context=IParagraphContainerTarget, layer=IAdminLayer,
                manager=IPropertiesMenu, permission=VIEW_SYSTEM_PERMISSION, weight=100)
class ParagraphsContainerMenu(MenuItem):
    """Paragraphs container menu"""

    label = _("Paragraphs...")
    icon_class = 'fa-paragraph'
    url = '#paragraphs.html'


#
# Paragraphs container view
#

@pagelet_config(name='paragraphs.html', context=IParagraphContainerTarget, layer=IPyAMSLayer, permission=VIEW_SYSTEM_PERMISSION)
@template_config(template='templates/container.pt', layer=IPyAMSLayer)
@implementer(IInnerPage)
class ParagraphContainerView(AdminView):
    """Paragraphs container view"""

    title = _("Paragraphs list")

    def __init__(self, context, request):
        super(ParagraphContainerView, self).__init__(context, request)
        self.table = ParagraphContainerTable(context, request, self)

    def update(self):
        super(ParagraphContainerView, self).update()
        self.table.update()


class ParagraphContainerTable(ProtectedFormObjectMixin, BaseTable):
    """Paragraphs container table"""

    id = 'paragraphs_list'
    hide_header = True
    sortOn = None

    def __init__(self, context, request, view):
        super(ParagraphContainerTable, self).__init__(context, request)
        self.view = view

    @property
    def cssClasses(self):
        classes = ['table', 'table-bordered', 'table-striped', 'table-hover', 'table-tight']
        permission = self.permission
        if (not permission) or self.request.has_permission(permission, self.context):
            classes.append('table-dnd')
        return {'table': ' '.join(classes)}

    @property
    def data_attributes(self):
        attributes = super(ParagraphContainerTable, self).data_attributes
        del attributes['tr']['data-ams-url']
        del attributes['tr']['data-toggle']
        attributes['table'] = {'id': self.id,
                               'data-ams-location': absolute_url(IParagraphContainer(self.context), self.request),
                               'data-ams-tablednd-drop-target': 'set-paragraphs-order.json'}
        return attributes

    @reify
    def values(self):
        return list(super(ParagraphContainerTable, self).values)

    def render(self):
        if not self.values:
            translate = self.request.localizer.translate
            return translate(_("No currently defined paragraph."))
        return super(ParagraphContainerTable, self).render()


@adapter_config(name='properties', context=(IParagraphContainerTarget, IPyAMSLayer, ParagraphContainerTable),
                provides=IColumn)
class ParagraphContainerPropertiesColumn(ActionColumn):
    """Paragraphs container properties column"""

    icon_class = 'fa fa-fw fa-edit'
    icon_hint = _("Paragraph properties")

    url = 'properties.html'
    modal_target = True

    weight = 5


@adapter_config(name='files', context=(IParagraphContainerTarget, IPyAMSLayer, ParagraphContainerTable),
                provides=IColumn)
class ParagraphContainerExtFileLinksColumn(ActionColumn):
    """Paragraphs container external files links column"""

    icon_class = 'fa fa-fw fa-file-text-o'
    icon_hint = _("External files")

    url = 'extfile-links.html'
    modal_target = True

    weight = 10

    def renderCell(self, item):
        if not IExtFileLinksContainerTarget.providedBy(item):
            return ''
        return super(ParagraphContainerExtFileLinksColumn, self).renderCell(item)


@adapter_config(name='links', context=(IParagraphContainerTarget, IPyAMSLayer, ParagraphContainerTable),
                provides=IColumn)
class ParagraphContainerLinkLinksColumn(ActionColumn):
    """Paragraphs container links links column"""

    icon_class = 'fa fa-fw fa-link'
    icon_hint = _("Useful links")

    url = 'link-links.html'
    modal_target = True

    weight = 15

    def renderCell(self, item):
        if not ILinkLinksContainerTarget.providedBy(item):
            return ''
        return super(ParagraphContainerLinkLinksColumn, self).renderCell(item)


@adapter_config(name='gallery', context=(IParagraphContainerTarget, IPyAMSLayer, ParagraphContainerTable),
                provides=IColumn)
class ParagraphContainerGalleryLinksColumn(ActionColumn):
    """Paragraphs container gallery links column"""

    icon_class = 'fa fa-fw fa-picture-o'
    icon_hint = _("Images galleries")

    url = 'gallery-links.html'
    modal_target = True

    weight = 20

    def renderCell(self, item):
        if not IGalleryLinksContainerTarget.providedBy(item):
            return ''
        return super(ParagraphContainerGalleryLinksColumn, self).renderCell(item)


@adapter_config(name='name', context=(IParagraphContainerTarget, IPyAMSLayer, ParagraphContainerTable),
                provides=IColumn)
class ParagraphContainerTitleColumn(I18nColumn, WfModifiedContentColumnMixin, GetAttrColumn):
    """Paragraph container title column"""

    _header = _("Title")

    weight = 50

    def getValue(self, obj):
        return II18n(obj).query_attribute('title', request=self.request) or '--'


@adapter_config(name='trash', context=(IParagraphContainerTarget, IPyAMSLayer, ParagraphContainerTable),
                provides=IColumn)
class ParagraphContainerTrashColumn(ProtectedFormObjectMixin, TrashColumn):
    """Paragraphs container trash column"""


@adapter_config(context=(IParagraphContainerTarget, IPyAMSLayer, ParagraphContainerTable), provides=IValues)
class ParagraphContainerValues(ContextRequestViewAdapter):
    """Paragraphs container values"""

    @property
    def values(self):
        return IParagraphContainer(self.context).values()


@adapter_config(context=(IParagraphContainerTarget, IPyAMSLayer, ParagraphContainerView), provides=IPageHeader)
class ParagraphHeaderAdapter(DefaultPageHeaderAdapter):
    """Paragraphs container header adapter"""

    back_url = '#properties.html'

    icon_class = 'fa fa-fw fa-paragraph'


@view_config(name='set-paragraphs-order.json', context=IParagraphContainerTarget, request_type=IPyAMSLayer,
             permission=MANAGE_CONTENT_PERMISSION, renderer='json', xhr=True)
def set_paragraphs_order(request):
    """Update paragraphs order"""
    container = IParagraphContainer(request.context)
    order = list(map(str, json.loads(request.params.get('names'))))
    container.updateOrder(order)
    return {'status': 'success'}
